@@ -22,27 +22,35 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+#include <mutex>
+
 #include "logger.hh"
 #include "misc.hh"
 #ifndef RECURSOR
 #include "statbag.hh"
 extern StatBag S;
 #endif
-#include "lock.hh"
 #include "namespaces.hh"
 
-pthread_once_t Logger::s_once;
-pthread_key_t Logger::s_loggerKey;
+thread_local Logger::PerThread Logger::t_perThread;
 
-Logger &theL(const string &pname)
+Logger& getLogger()
 {
-  static Logger l("", LOG_DAEMON);
-  if(!pname.empty())
-    l.setName(pname);
-  return l;
+  /* Since the Logger can be called very early, we need to make sure
+     that the relevant parts are initialized no matter what, which is tricky
+     because we can't easily control the initialization order, especially with
+     built-in backends.
+     t_perThread is thread_local, so it will be initialized when first accessed,
+     but we need to make sure that the object itself is initialized, and making
+     it a function-level static variable achieves that, because it will be
+     initialized the first time we enter this function at the very last.
+  */
+  static Logger log("", LOG_DAEMON);
+  return log;
 }
 
-void Logger::log(const string &msg, Urgency u)
+void Logger::log(const string &msg, Urgency u) noexcept
 {
 #ifndef RECURSOR
   bool mustAccount(false);
@@ -53,13 +61,46 @@ void Logger::log(const string &msg, Urgency u)
       struct tm tm;
       time_t t;
       time(&t);
-      tm=*localtime(&t);
+      localtime_r(&t, &tm);
       strftime(buffer,sizeof(buffer),"%b %d %H:%M:%S ", &tm);
     }
 
-    static pthread_mutex_t m = PTHREAD_MUTEX_INITIALIZER;
-    Lock l(&m); // the C++-2011 spec says we need this, and OSX actually does
-    clog << string(buffer) + msg <<endl;
+    string prefix;
+    if (d_prefixed) {
+      switch(u) {
+        case All:
+          prefix = "[all] ";
+          break;
+        case Alert:
+          prefix = "[ALERT] ";
+          break;
+        case Critical:
+          prefix = "[CRITICAL] ";
+          break;
+        case Error:
+          prefix = "[ERROR] ";
+          break;
+        case Warning:
+          prefix = "[WARNING] ";
+          break;
+        case Notice:
+          prefix = "[NOTICE] ";
+          break;
+        case Info:
+          prefix = "[INFO] ";
+          break;
+        case Debug:
+          prefix = "[DEBUG] ";
+          break;
+        case None:
+          prefix = "[none] ";
+          break;
+      }
+    }
+
+    static std::mutex m;
+    std::lock_guard<std::mutex> l(m); // the C++-2011 spec says we need this, and OSX actually does
+    clog << string(buffer) + prefix + msg <<endl;
 #ifndef RECURSOR
     mustAccount=true;
 #endif
@@ -72,8 +113,14 @@ void Logger::log(const string &msg, Urgency u)
   }
 
 #ifndef RECURSOR
-  if(mustAccount)
-    S.ringAccount("logmessages",msg);
+  if(mustAccount) {
+      try {
+        S.ringAccount("logmessages",msg);
+      }
+      catch (const runtime_error& e) {
+        cerr << e.what() << endl;
+      }
+  }
 #endif
 }
 
@@ -102,58 +149,29 @@ void Logger::setName(const string &_name)
   open();
 }
 
-void Logger::initKey()
+Logger::Logger(string n, int facility) :
+  name(std::move(n)), flags(LOG_PID|LOG_NDELAY), d_facility(facility), d_loglevel(Logger::None),
+  consoleUrgency(Error), opened(false), d_disableSyslog(false)
 {
-  if(pthread_key_create(&s_loggerKey, perThreadDestructor))
-    unixDie("Creating thread key for logger");
-}
-
-Logger::Logger(const string &n, int facility)
-{
-  opened=false;
-  flags=LOG_PID|LOG_NDELAY;
-  d_facility=facility;
-  d_loglevel=Logger::None;
-  d_disableSyslog=false;
-  consoleUrgency=Error;
-  name=n;
-
-  if(pthread_once(&s_once, initKey))
-    unixDie("Creating thread key for logger");
-
   open();
 
 }
 
 Logger& Logger::operator<<(Urgency u)
 {
-  getPerThread()->d_urgency=u;
+  getPerThread().d_urgency=u;
   return *this;
 }
 
-void Logger::perThreadDestructor(void* buf)
+Logger::PerThread& Logger::getPerThread()
 {
-  PerThread* pt = (PerThread*) buf;
-  delete pt;
-}
-
-Logger::PerThread* Logger::getPerThread()
-{
-  void *buf=pthread_getspecific(s_loggerKey);
-  PerThread* ret;
-  if(buf)
-    ret = (PerThread*) buf;
-  else {
-    ret = new PerThread();
-    pthread_setspecific(s_loggerKey, (void*)ret);
-  }
-  return ret;
+  return t_perThread;
 }
 
 Logger& Logger::operator<<(const string &s)
 {
-  PerThread* pt =getPerThread();
-  pt->d_output.append(s);
+  PerThread& pt = getPerThread();
+  pt.d_output.append(s);
   return *this;
 }
 
@@ -163,71 +181,13 @@ Logger& Logger::operator<<(const char *s)
   return *this;
 }
 
-Logger& Logger::operator<<(int i)
-{
-  ostringstream tmp;
-  tmp<<i;
-
-  *this<<tmp.str();
-
-  return *this;
-}
-
-Logger& Logger::operator<<(double i)
-{
-  ostringstream tmp;
-  tmp<<i;
-  *this<<tmp.str();
-  return *this;
-}
-
-Logger& Logger::operator<<(unsigned int i)
-{
-  ostringstream tmp;
-  tmp<<i;
-
-  *this<<tmp.str();
-
-  return *this;
-}
-
-Logger& Logger::operator<<(unsigned long i)
-{
-  ostringstream tmp;
-  tmp<<i;
-
-  *this<<tmp.str();
-
-  return *this;
-}
-
-Logger& Logger::operator<<(unsigned long long i)
-{
-  ostringstream tmp;
-  tmp<<i;
-
-  *this<<tmp.str();
-
-  return *this;
-}
-
-Logger& Logger::operator<<(long i)
-{
-  ostringstream tmp;
-  tmp<<i;
-
-  *this<<tmp.str();
-
-  return *this;
-}
-
 Logger& Logger::operator<<(ostream & (&)(ostream &))
 {
-  PerThread* pt =getPerThread();
+  PerThread& pt = getPerThread();
 
-  log(pt->d_output, pt->d_urgency);
-  pt->d_output.clear();
-  pt->d_urgency=Info;
+  log(pt.d_output, pt.d_urgency);
+  pt.d_output.clear();
+  pt.d_urgency=Info;
   return *this;
 }
 
@@ -240,7 +200,7 @@ Logger& Logger::operator<<(const DNSName &d)
 
 Logger& Logger::operator<<(const ComboAddress &ca)
 {
-  *this<<ca.toString();
+  *this<<ca.toLogString();
   return *this;
 }
 
