@@ -1,14 +1,16 @@
 #define BOOST_TEST_DYN_LINK
 #define BOOST_TEST_NO_MAIN
 #include <boost/test/unit_test.hpp>
-#include <boost/assign/std/map.hpp>
+
+#include <cmath>
 #include <numeric>
-#include <math.h>
+#include <unordered_set>
+
 #include "dnsname.hh"
 #include "misc.hh"
 #include "dnswriter.hh"
 #include "dnsrecords.hh"
-#include <unordered_set>
+
 using namespace boost;
 using std::string;
 
@@ -24,7 +26,7 @@ BOOST_AUTO_TEST_CASE(test_basic) {
 
   string before("www.ds9a.nl.");
   DNSName b(before);
-  BOOST_CHECK_EQUAL(b.getRawLabels().size(), 3);
+  BOOST_CHECK_EQUAL(b.getRawLabels().size(), 3U);
   string after(b.toString());
   BOOST_CHECK_EQUAL(before, after);
 
@@ -145,7 +147,7 @@ BOOST_AUTO_TEST_CASE(test_basic) {
   auto labels=rfc4343_2_2.getRawLabels();
   BOOST_CHECK_EQUAL(*labels.begin(), "Donald E. Eastlake 3rd");
   BOOST_CHECK_EQUAL(*labels.rbegin(), "example");
-  BOOST_CHECK_EQUAL(labels.size(), 2);
+  BOOST_CHECK_EQUAL(labels.size(), 2U);
 
   DNSName build;
   build.appendRawLabel("Donald E. Eastlake 3rd");
@@ -174,14 +176,14 @@ BOOST_AUTO_TEST_CASE(test_basic) {
 
 BOOST_AUTO_TEST_CASE(test_trim) {
   DNSName w("www.powerdns.com.");
-  BOOST_CHECK_EQUAL(w.countLabels(), 3);
+  BOOST_CHECK_EQUAL(w.countLabels(), 3U);
   w.trimToLabels(2);
   BOOST_CHECK_EQUAL(w.toString(), "powerdns.com.");
   DNSName w2("powerdns.com.");
   BOOST_CHECK(w==w2);
 
   DNSName root(".");
-  BOOST_CHECK_EQUAL(root.countLabels(), 0);
+  BOOST_CHECK_EQUAL(root.countLabels(), 0U);
 }
 
 BOOST_AUTO_TEST_CASE(test_toolong) {
@@ -370,11 +372,11 @@ BOOST_AUTO_TEST_CASE(test_hashContainer) {
   std::unordered_set<DNSName> s;
   s.insert(DNSName("www.powerdns.com"));
   BOOST_CHECK(s.count(DNSName("WwW.PoWerDNS.CoM")));
-  BOOST_CHECK_EQUAL(s.size(), 1);
+  BOOST_CHECK_EQUAL(s.size(), 1U);
   s.insert(DNSName("www.POWERDNS.com"));
-  BOOST_CHECK_EQUAL(s.size(), 1);
+  BOOST_CHECK_EQUAL(s.size(), 1U);
   s.insert(DNSName("www2.POWERDNS.com"));
-  BOOST_CHECK_EQUAL(s.size(), 2);
+  BOOST_CHECK_EQUAL(s.size(), 2U);
 
   s.clear();
   unsigned int n=0;
@@ -523,9 +525,10 @@ BOOST_AUTO_TEST_CASE(test_suffixmatch) {
 BOOST_AUTO_TEST_CASE(test_suffixmatch_tree) {
   SuffixMatchTree<DNSName> smt;
   DNSName ezdns("ezdns.it.");
-  smt.add(ezdns, ezdns);
+  smt.add(ezdns, DNSName(ezdns));
 
-  smt.add(DNSName("org.").getRawLabels(), DNSName("org."));
+  auto labels = DNSName("org.").getRawLabels();
+  smt.add(labels, DNSName("org."));
 
   DNSName wwwpowerdnscom("www.powerdns.com.");
   DNSName wwwezdnsit("www.ezdns.it.");
@@ -548,14 +551,14 @@ BOOST_AUTO_TEST_CASE(test_suffixmatch_tree) {
   BOOST_CHECK(smt.lookup(DNSName("images.bbc.co.uk.")) == nullptr);
   BOOST_CHECK(smt.lookup(DNSName("www.news.gov.uk.")) == nullptr);
 
-  smt.add(g_rootdnsname, g_rootdnsname); // block the root
+  smt.add(g_rootdnsname, DNSName(g_rootdnsname)); // block the root
   BOOST_REQUIRE(smt.lookup(DNSName("a.root-servers.net.")));
   BOOST_CHECK_EQUAL(*smt.lookup(DNSName("a.root-servers.net.")), g_rootdnsname);
 
   DNSName apowerdnscom("a.powerdns.com.");
   DNSName bpowerdnscom("b.powerdns.com.");
-  smt.add(apowerdnscom, apowerdnscom);
-  smt.add(bpowerdnscom, bpowerdnscom);
+  smt.add(apowerdnscom, DNSName(apowerdnscom));
+  smt.add(bpowerdnscom, DNSName(bpowerdnscom));
   BOOST_REQUIRE(smt.lookup(apowerdnscom));
   BOOST_CHECK_EQUAL(*smt.lookup(apowerdnscom), apowerdnscom);
   BOOST_REQUIRE(smt.lookup(bpowerdnscom));
@@ -563,8 +566,8 @@ BOOST_AUTO_TEST_CASE(test_suffixmatch_tree) {
 
   DNSName examplenet("example.net.");
   DNSName net("net.");
-  smt.add(examplenet, examplenet);
-  smt.add(net, net);
+  smt.add(examplenet, DNSName(examplenet));
+  smt.add(net, DNSName(net));
   BOOST_REQUIRE(smt.lookup(examplenet));
   BOOST_CHECK_EQUAL(*smt.lookup(examplenet), examplenet);
   BOOST_REQUIRE(smt.lookup(net));
@@ -577,10 +580,10 @@ BOOST_AUTO_TEST_CASE(test_suffixmatch_tree) {
   BOOST_CHECK_EQUAL(*smt.lookup(examplenet), examplenet);
 
   smt = SuffixMatchTree<DNSName>();
-  smt.add(examplenet, examplenet);
-  smt.add(net, net);
+  smt.add(examplenet, DNSName(examplenet));
+  smt.add(net, DNSName(net));
   smt.add(DNSName("news.bbc.co.uk."), DNSName("news.bbc.co.uk."));
-  smt.add(apowerdnscom, apowerdnscom);
+  smt.add(apowerdnscom, DNSName(apowerdnscom));
 
   smt.remove(DNSName("not-such-entry.news.bbc.co.uk."));
   BOOST_REQUIRE(smt.lookup(DNSName("news.bbc.co.uk.")));
@@ -596,8 +599,8 @@ BOOST_AUTO_TEST_CASE(test_suffixmatch_tree) {
   BOOST_CHECK(smt.lookup(net) == nullptr);
   BOOST_CHECK(smt.lookup(examplenet) == nullptr);
 
-  smt.add(examplenet, examplenet);
-  smt.add(net, net);
+  smt.add(examplenet, DNSName(examplenet));
+  smt.add(net, DNSName(net));
   BOOST_REQUIRE(smt.lookup(examplenet));
   BOOST_CHECK_EQUAL(*smt.lookup(examplenet), examplenet);
   BOOST_REQUIRE(smt.lookup(net));
@@ -615,21 +618,21 @@ BOOST_AUTO_TEST_CASE(test_suffixmatch_tree) {
   smt.remove(net);
 
   size_t count = 0;
-  smt.visit([apowerdnscom, &count](const SuffixMatchTree<DNSName>& smt) {
+  smt.visit([apowerdnscom, &count](const SuffixMatchTree<DNSName>& smtarg) {
       count++;
-      BOOST_CHECK_EQUAL(smt.d_value, apowerdnscom);
+      BOOST_CHECK_EQUAL(smtarg.d_value, apowerdnscom);
     });
-  BOOST_CHECK_EQUAL(count, 1);
+  BOOST_CHECK_EQUAL(count, 1U);
 
   BOOST_CHECK_EQUAL(*smt.lookup(apowerdnscom), apowerdnscom);
   smt.remove(apowerdnscom);
   BOOST_CHECK(smt.lookup(apowerdnscom) == nullptr);
 
   count = 0;
-  smt.visit([&count](const SuffixMatchTree<DNSName>& smt) {
+  smt.visit([&count](const SuffixMatchTree<DNSName>&) {
       count++;
     });
-  BOOST_CHECK_EQUAL(count, 0);
+  BOOST_CHECK_EQUAL(count, 0U);
 }
 
 
@@ -758,20 +761,20 @@ BOOST_AUTO_TEST_CASE(test_name_length_max) { // 255 char name
   { // append
     DNSName dn(name);
     dn.appendRawLabel(label);
-    BOOST_CHECK_EQUAL(dn.toString().size(), 254);
+    BOOST_CHECK_EQUAL(dn.toString().size(), 254U);
   }
 
   { // prepend
     DNSName dn(name);
     dn.prependRawLabel(label);
-    BOOST_CHECK_EQUAL(dn.toString().size(), 254);
+    BOOST_CHECK_EQUAL(dn.toString().size(), 254U);
   }
 
   { // concat
     DNSName dn(name);
 
     dn += DNSName(label + ".");
-    BOOST_CHECK_EQUAL(dn.toString().size(), 254);
+    BOOST_CHECK_EQUAL(dn.toString().size(), 254U);
   }
 }
 
@@ -919,15 +922,15 @@ BOOST_AUTO_TEST_CASE(test_compression_loop2) { // Compression loop (deep recursi
 
 BOOST_AUTO_TEST_CASE(test_wirelength) { // Testing if we get the correct value from the wirelength function
   DNSName name("www.powerdns.com");
-  BOOST_CHECK_EQUAL(name.wirelength(), 18);
+  BOOST_CHECK_EQUAL(name.wirelength(), 18U);
 
   DNSName sname("powerdns.com");
   sname.prependRawLabel(string("ww\x00""w", 4));
-  BOOST_CHECK_EQUAL(sname.wirelength(), 19);
+  BOOST_CHECK_EQUAL(sname.wirelength(), 19U);
 
   sname = DNSName("powerdns.com");
   sname.prependRawLabel(string("www\x00", 4));
-  BOOST_CHECK_EQUAL(sname.wirelength(), 19);
+  BOOST_CHECK_EQUAL(sname.wirelength(), 19U);
 }
 
 BOOST_AUTO_TEST_CASE(test_getrawlabel) {
