@@ -69,6 +69,30 @@ BOOST_AUTO_TEST_CASE(test_get_entry)
   BOOST_CHECK_EQUAL(ne.d_auth, auth);
 }
 
+BOOST_AUTO_TEST_CASE(test_get_entry2038)
+{
+  /* Add a full name negative entry to the cache and attempt to get an entry for
+   * the A record. Should yield the full name does not exist entry
+   */
+  DNSName qname("www2.powerdns.com");
+  DNSName auth("powerdns.com");
+
+  timeval now{INT_MAX - 300, 0};
+
+  NegCache cache;
+  cache.add(genNegCacheEntry(qname, auth, now));
+
+  BOOST_CHECK_EQUAL(cache.size(), 1U);
+
+  NegCache::NegCacheEntry ne;
+  bool ret = cache.get(qname, QType(QType::A), now, ne);
+
+  BOOST_CHECK(ret);
+  BOOST_CHECK_EQUAL(ne.d_name, qname);
+  BOOST_CHECK_EQUAL(ne.d_qtype.getName(), QType(0).getName());
+  BOOST_CHECK_EQUAL(ne.d_auth, auth);
+}
+
 BOOST_AUTO_TEST_CASE(test_get_entry_exact_type)
 {
   /* Add a full name negative entry to the cache and attempt to get an entry for
@@ -414,7 +438,7 @@ BOOST_AUTO_TEST_CASE(test_clear)
 
 BOOST_AUTO_TEST_CASE(test_dumpToFile)
 {
-  NegCache cache;
+  NegCache cache(1);
   vector<string> expected;
   expected.push_back("www1.powerdns.com. 600 IN TYPE0 VIA powerdns.com. ; (Indeterminate)\n");
   expected.push_back("powerdns.com. 600 IN SOA ns1. hostmaster. 1 2 3 4 5 ; (Indeterminate)\n");
@@ -437,26 +461,25 @@ BOOST_AUTO_TEST_CASE(test_dumpToFile)
   if (!fp)
     BOOST_FAIL("Temporary file could not be opened");
 
-  cache.dumpToFile(fp.get());
+  cache.dumpToFile(fp.get(), now);
 
   rewind(fp.get());
   char* line = nullptr;
   size_t len = 0;
   ssize_t read;
 
-  for (const auto& str : expected) {
+  for (auto str : expected) {
     read = getline(&line, &len, fp.get());
     if (read == -1)
       BOOST_FAIL("Unable to read a line from the temp file");
-    BOOST_CHECK_EQUAL(line, str);
+    // The clock might have ticked so the 600 becomes 599
+    BOOST_CHECK(line == str);
   }
 
-  if (line != nullptr) {
-    /* getline() allocates a buffer when called with a nullptr,
-       then reallocates it when needed, but we need to free the
-       last allocation if any. */
-    free(line);
-  }
+  /* getline() allocates a buffer when called with a nullptr,
+     then reallocates it when needed, but we need to free the
+     last allocation if any. */
+  free(line);
 }
 
 BOOST_AUTO_TEST_CASE(test_count)
